@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  gnome-print-preview.c: print preview driver
  *
@@ -25,37 +25,47 @@
  */
 
 #include <config.h>
+#include "gnome-print-preview.h"
 
 #include <string.h>
 #include <math.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include <libgnomecanvas/gnome-canvas-clipgroup.h>
+
+#include <libgnomeprint/private/gnome-print-private.h>
 #include <libgnomeprint/private/gp-gc-private.h>
 #include <libgnomeprint/private/gnome-glyphlist-private.h>
 
 #include "gnome-canvas-hacktext.h"
-#include "gnome-print-preview-private.h"
 
-struct _GnomePrintPreviewPrivate {
-	GPGC * gc;
+typedef struct _GnomePrintPreviewPrivate GnomePrintPreviewPrivate;
 
-	/* Current page displayed */
-	int top_page;
-	int current_page;
+struct _GnomePrintPreview {
+	GnomePrintContext pc;
 
-	/* The root group, with a translation setup */
-	GnomeCanvasItem *root;
+	GPtrArray *groups;
+	GnomeCanvasGroup *group;
+	guint pages;
 
-	/* The current page */
-	GnomeCanvasItem *page;
+	/* Properties */
+	GnomeCanvasGroup *page;
+	gboolean theme_compliance;
+	gboolean use_theme;
+	gboolean only_first;
+};
 
-	/* Strict theme compliance [#96802] */
-	gboolean theme_compliance; 
+struct _GnomePrintPreviewClass {
+	GnomePrintContextClass parent_class;
 };
 
 static GnomePrintContextClass *parent_class;
-static gboolean use_theme;
+
+#define GPP_COLOR_RGBA(color, ALPHA) \
+	((guint32) (ALPHA              | \
+	 (((color).red   / 256) << 24) | \
+	 (((color).green / 256) << 16) | \
+	 (((color).blue  / 256) << 8)))
 
 static void
 outline_set_style_cb (GtkWidget *canvas, GnomeCanvasItem *item)
@@ -68,29 +78,20 @@ outline_set_style_cb (GtkWidget *canvas, GnomeCanvasItem *item)
 	
 	gnome_canvas_item_set (item, "outline_color_rgba", color, NULL);
 }
-	
 
 static int
-gpp_stroke (GnomePrintContext *pc, const ArtBpath *bpath)
+gnome_print_preview_stroke (GnomePrintContext *pc, const ArtBpath *bpath)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomePrintPreviewPrivate *priv = pp->priv;
-	GnomeCanvasGroup *group;
 	GnomeCanvasItem *item;
 	GnomeCanvasPathDef *path;
 
-	/* fixme: currentpath invariants */
-
-	group = (GnomeCanvasGroup *) gp_gc_get_data (priv->gc);
-	g_assert (group != NULL);
-	g_assert (GNOME_IS_CANVAS_GROUP (group));
+	if (pp->only_first && (pp->pages > 1))
+		return GNOME_PRINT_OK;
 
 	path = gnome_canvas_path_def_new_from_foreign_bpath ((ArtBpath *) bpath);
 
-	/* Fixme: Can we assume that linewidth == width_units? */
-	/* Probably yes, as object->page ctm == identity */
-
-	item = gnome_canvas_item_new (group,
+	item = gnome_canvas_item_new (pp->group,
 		gnome_canvas_bpath_get_type (),
 		"bpath",	path,
 		"width_units",	gp_gc_get_linewidth (pc->gc),
@@ -103,7 +104,7 @@ gpp_stroke (GnomePrintContext *pc, const ArtBpath *bpath)
 
 	gnome_canvas_path_def_unref (path);
 
-	if (use_theme)
+	if (pp->use_theme)
 		outline_set_style_cb (GTK_WIDGET (item->canvas), item);	
 	return 1;
 }
@@ -121,24 +122,18 @@ fill_set_style_cb (GtkWidget *canvas, GnomeCanvasItem *item)
 }
 	
 static int
-gpp_fill (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
+gnome_print_preview_fill (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
 {
-	GnomePrintPreview *pp;
-	GnomePrintPreviewPrivate * priv;
-	GnomeCanvasGroup *group;
+	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
 	GnomeCanvasItem *item;
 	GnomeCanvasPathDef *path;
-	
-	pp = GNOME_PRINT_PREVIEW (pc);
-	priv = pp->priv;
 
-	group = (GnomeCanvasGroup *) gp_gc_get_data (priv->gc);
-	g_assert (group != NULL);
-	g_assert (GNOME_IS_CANVAS_GROUP (group));
+	if (pp->only_first && (pp->pages > 1))
+		return GNOME_PRINT_OK;
 
 	path = gnome_canvas_path_def_new_from_foreign_bpath ((ArtBpath *) bpath);
 
-	item = gnome_canvas_item_new (group,
+	item = gnome_canvas_item_new (pp->group,
 		gnome_canvas_bpath_get_type (),
 		"bpath", path,
 		"outline_color", NULL,
@@ -147,34 +142,49 @@ gpp_fill (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
 		NULL);
 	gnome_canvas_path_def_unref (path);
 
-	if (use_theme)
+	if (pp->use_theme)
 		fill_set_style_cb (GTK_WIDGET (item->canvas), item);
 	return 1;
 }
 
 static int
-gpp_clip (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
+gnome_print_preview_gsave (GnomePrintContext *pc)
 {
-	GnomePrintPreview *pp;
-	GnomePrintPreviewPrivate * priv;
-	GnomeCanvasGroup * group;
-	GnomeCanvasItem * clip;
+	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
+
+	if (!pp->groups)
+		pp->groups = g_ptr_array_new ();
+	g_ptr_array_add (pp->groups, pp->group);
+	pp->group = GNOME_CANVAS_GROUP (
+		gnome_canvas_item_new (pp->page, GNOME_TYPE_CANVAS_GROUP, NULL));
+	return GNOME_PRINT_OK;
+}
+
+static int
+gnome_print_preview_grestore (GnomePrintContext *pc)
+{
+	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
+
+	pp->group = g_ptr_array_remove_index (pp->groups, pp->groups->len - 1);
+	return GNOME_PRINT_OK;
+}
+
+static int
+gnome_print_preview_clip (GnomePrintContext *pc, const ArtBpath *b, ArtWindRule rule)
+{
+	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
 	GnomeCanvasPathDef *path;
 
-	pp = GNOME_PRINT_PREVIEW (pc);
-	priv = pp->priv;
+	if (pp->only_first && (pp->pages > 1))
+		return GNOME_PRINT_OK;
 
-	group = (GnomeCanvasGroup *) gp_gc_get_data (priv->gc);
+	path = gnome_canvas_path_def_new_from_foreign_bpath ((ArtBpath *) b);
 
-	path = gnome_canvas_path_def_new_from_foreign_bpath ((ArtBpath *) bpath);
-
-	clip = gnome_canvas_item_new (group,
+	pp->group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (pp->group,
 		gnome_canvas_clipgroup_get_type (),
 		"path", path,
 		"wind", rule,
-		NULL);
-
-	gp_gc_set_data (priv->gc, clip);
+		NULL));
 
 	gnome_canvas_path_def_unref (path);
 
@@ -182,22 +192,24 @@ gpp_clip (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
 }
 
 static void
-gpp_image_free_pix (guchar *pixels, gpointer data)
+gnome_print_preview_image_free_pix (guchar *pixels, gpointer data)
 {
 	g_free (pixels);
 
 }
 
 static int
-gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint w, gint h, gint rowstride, gint ch)
+gnome_print_preview_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint w, gint h, gint rowstride, gint ch)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomeCanvasGroup * group;
 	GnomeCanvasItem *item;
 	GdkPixbuf *pixbuf;
 	int size, bpp;
 	void *dup;
-	
+
+	if (pp->only_first && (pp->pages > 1))
+		return GNOME_PRINT_OK;
+
 	/*
 	 * We do convert gray scale images to RGB
 	 */
@@ -216,14 +228,14 @@ gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint
 		memcpy (dup, px, size);
 		pixbuf = gdk_pixbuf_new_from_data (dup, GDK_COLORSPACE_RGB,
 						   FALSE, 8, w, h, rowstride,
-						   gpp_image_free_pix, NULL);
+						   gnome_print_preview_image_free_pix, NULL);
 	} else if (ch == 4) {
 		memcpy (dup, px, size);
 		pixbuf = gdk_pixbuf_new_from_data (dup, GDK_COLORSPACE_RGB,
 				                   TRUE, 8, w, h, rowstride,
-						   gpp_image_free_pix, NULL);
+						   gnome_print_preview_image_free_pix, NULL);
 	} else if (ch == 1) {
-		const char *source;
+		unsigned char const *source;
 		char *target;
 		int  ix, iy;
 
@@ -240,13 +252,11 @@ gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint
 		}
 		pixbuf = gdk_pixbuf_new_from_data (dup, GDK_COLORSPACE_RGB,
 						   FALSE, 8, w, h, rowstride * 3,
-						   gpp_image_free_pix, NULL);
+						   gnome_print_preview_image_free_pix, NULL);
 	} else
 		return -1;
 
-	group = (GnomeCanvasGroup *) gp_gc_get_data (pp->priv->gc);
-
-	item = gnome_canvas_item_new (group,
+	item = gnome_canvas_item_new (pp->group,
 				      GNOME_TYPE_CANVAS_PIXBUF,
 				      "pixbuf", pixbuf,
 				      "x",      0.0,
@@ -275,15 +285,43 @@ gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint
 	return 1;
 }
 
-static int
-gpp_showpage (GnomePrintContext *pc)
+static void
+gnome_print_preview_clear (GnomePrintPreview *pp)
 {
-	return GNOME_PRINT_OK;
+	GnomeCanvasGroup *g;
+
+	g_return_if_fail (GNOME_IS_PRINT_PREVIEW (pp));
+
+	if (!pp->page)
+		return;
+	g = GNOME_CANVAS_GROUP (pp->page);
+	while (g->item_list) {
+		GnomeCanvasItem *i = g->item_list->data;
+		g->item_list = g_list_delete_link (g->item_list, g->item_list);
+		gtk_object_destroy (GTK_OBJECT (i));
+	}
+	pp->group = pp->page;
+}
+
+void
+gnome_print_preview_reset (GnomePrintPreview *pp)
+{
+	g_return_if_fail (GNOME_IS_PRINT_PREVIEW (pp));
+
+	pp->pages = 0;
+	gnome_print_preview_clear (pp);
 }
 
 static int
-gpp_beginpage (GnomePrintContext *pc, const guchar *name)
+gnome_print_preview_beginpage (GnomePrintContext *pc, const guchar *name)
 {
+	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
+
+	pp->pages++;
+	if (pp->only_first && (pp->pages > 1))
+		return GNOME_PRINT_OK;
+	gnome_print_preview_clear (pp);
+
 	return GNOME_PRINT_OK;
 }
 
@@ -306,27 +344,23 @@ glyphlist_set_style_cb (GtkWidget *canvas, GnomeCanvasItem *item)
 		}
 	}
 	gnome_canvas_item_set (item, "glyphlist", new, NULL);
+	gnome_glyphlist_unref (new);
 }
 
 static int
-gpp_glyphlist (GnomePrintContext *pc, const gdouble *affine, GnomeGlyphList * glyphlist)
+gnome_print_preview_glyphlist (GnomePrintContext *pc, const gdouble *affine, GnomeGlyphList * glyphlist)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
-	GnomeCanvasGroup *group;
 	GnomeCanvasItem *item;
 	double transform[6], a[6];
 
-	/*
-	 * The X and Y positions were computed to be the base
-	 * with the translation already done for the new
-	 * Postscript->Canvas translation
-	 */
-	memcpy (transform, gp_gc_get_ctm (pc->gc), sizeof (transform));
+	if (pp->only_first && (pp->pages > 1))
+		return GNOME_PRINT_OK;
+
 	art_affine_scale (a, 1.0, -1.0);
 	art_affine_multiply (transform, a, affine);
 
-	group = (GnomeCanvasGroup *) gp_gc_get_data (pp->priv->gc);
-	item = gnome_canvas_item_new (group,
+	item = gnome_canvas_item_new (pp->group,
 				      gnome_canvas_hacktext_get_type (),
 				      "x", 0.0,
 				      "y", 0.0,
@@ -335,63 +369,88 @@ gpp_glyphlist (GnomePrintContext *pc, const gdouble *affine, GnomeGlyphList * gl
 
 	gnome_canvas_item_affine_absolute (item, transform);
 
-	if (use_theme)
+	if (pp->use_theme)
 		glyphlist_set_style_cb (GTK_WIDGET (item->canvas), item);	
-	return 0;
-}
-
-static int
-gpp_gsave (GnomePrintContext *ctx)
-{
-	GnomePrintPreview *pp;
-
-	pp = GNOME_PRINT_PREVIEW (ctx);
-
-	gp_gc_gsave (pp->priv->gc);
-
 	return GNOME_PRINT_OK;
 }
 
-static int
-gpp_grestore (GnomePrintContext *ctx)
-{
-	GnomePrintPreview *pp;
-
-	pp = GNOME_PRINT_PREVIEW (ctx);
-
-	gp_gc_grestore (pp->priv->gc);
-
-	return GNOME_PRINT_OK;
-}
-
-static int
-gpp_close (GnomePrintContext *pc)
-{
-	return 0;
-}
-
 static void
-gpp_finalize (GObject *object)
+gnome_print_preview_finalize (GObject *object)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (object);
-	GnomePrintPreviewPrivate *priv = pp->priv;
 
-	gp_gc_unref (priv->gc);
-
-	if (pp->canvas)
-		g_object_unref (G_OBJECT (pp->canvas));
-
-	if (priv->page)
-		gtk_object_destroy (GTK_OBJECT (priv->page));
-
-	if (priv->root)
-		gtk_object_destroy (GTK_OBJECT (priv->root));
-	
-	g_free (priv);
+	if (pp->groups) {
+		g_ptr_array_free (pp->groups, TRUE);
+		pp->groups = NULL;
+	}
+	if (pp->page) {
+		g_object_unref (G_OBJECT (pp->page));
+		pp->page = NULL;
+	}
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+enum {
+	PROP_0,
+	PROP_GROUP,
+	PROP_USE_THEME,
+	PROP_THEME_COMPLIANCE,
+	PROP_ONLY_FIRST
+};
+
+static void
+gnome_print_preview_get_property (GObject *gobject, guint param_id,
+		  GValue *value, GParamSpec *pspec)
+{
+	GnomePrintPreview *preview = GNOME_PRINT_PREVIEW (gobject);
+	
+	switch (param_id) {
+	case PROP_GROUP:
+		g_value_set_object (value, preview->page);
+		break;
+	case PROP_THEME_COMPLIANCE:
+		g_value_set_boolean (value, preview->theme_compliance);
+		break;
+	case PROP_USE_THEME:
+		g_value_set_boolean (value, preview->use_theme);
+		break;
+	case PROP_ONLY_FIRST:
+		g_value_set_boolean (value, preview->only_first);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, param_id, pspec);
+	}
+}
+
+static void
+gnome_print_preview_set_property (GObject *gobject, guint param_id,
+		  const GValue *value, GParamSpec *pspec)
+{
+	GnomePrintPreview *preview = GNOME_PRINT_PREVIEW (gobject);
+
+	switch (param_id) {
+	case PROP_GROUP:
+		if (preview->page)
+			g_object_unref (G_OBJECT (preview->page));
+		preview->page = g_value_get_object (value);
+		if (preview->page)
+			g_object_ref (G_OBJECT (preview->page));
+		break;
+	case PROP_THEME_COMPLIANCE:
+		preview->theme_compliance = g_value_get_boolean (value);
+		break;
+	case PROP_USE_THEME:
+		preview->use_theme = g_value_get_boolean (value);
+		break;
+	case PROP_ONLY_FIRST:
+		preview->only_first = g_value_get_boolean (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, param_id, pspec);
+	}
+}
+
 static void
 class_init (GnomePrintPreviewClass *klass)
 {
@@ -400,109 +459,57 @@ class_init (GnomePrintPreviewClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
-	object_class->finalize = gpp_finalize;
-	
-	pc_class->beginpage = gpp_beginpage;
-	pc_class->showpage  = gpp_showpage;
-	pc_class->clip      = gpp_clip;
-	pc_class->fill      = gpp_fill;
-	pc_class->stroke    = gpp_stroke;
-	pc_class->image     = gpp_image;
-	pc_class->glyphlist = gpp_glyphlist;
-	pc_class->gsave     = gpp_gsave;
-	pc_class->grestore  = gpp_grestore;
-	pc_class->close     = gpp_close;
-}
+	/* Set up the print context class */
+	pc_class->beginpage = gnome_print_preview_beginpage;
+	pc_class->clip      = gnome_print_preview_clip;
+	pc_class->fill      = gnome_print_preview_fill;
+	pc_class->stroke    = gnome_print_preview_stroke;
+	pc_class->image     = gnome_print_preview_image;
+	pc_class->glyphlist = gnome_print_preview_glyphlist;
+	pc_class->gsave     = gnome_print_preview_gsave;
+	pc_class->grestore  = gnome_print_preview_grestore;
 
-static void
-instance_init (GnomePrintPreview *preview)
-{
-	preview->priv = g_new0 (GnomePrintPreviewPrivate, 1);
-
-	preview->priv->gc = gp_gc_new ();
-}
-
-static void
-clear_val (GtkObject *object, void **val)
-{
-	*val = NULL;
-}
-
-void
-gnome_print_preview_set_use_theme (gboolean theme)
-{
-	use_theme = theme;
-}
-gboolean
-gnome_print_preview_get_use_theme ()
-{
-	return use_theme;
-}
-
-/**
- * gnome_print_preview_theme_compliance:
- * @pp: 
- * @compliance: 
- * 
- * This has to go away for GNOME 2.4. Make compliance an argument of
- * gnome_print_preview_new_full
- **/
-void
-gnome_print_preview_theme_compliance (GnomePrintPreview *pp,
-				      gboolean compliance)
-{
-	pp->priv->theme_compliance = compliance;
+	/* Set upt the object class */
+	object_class->finalize = gnome_print_preview_finalize;
+	object_class->get_property = gnome_print_preview_get_property;
+	object_class->set_property = gnome_print_preview_set_property;
+	g_object_class_install_property (G_OBJECT_CLASS (object_class),
+		PROP_GROUP, g_param_spec_object ("group",
+			"Group", "Group", GNOME_TYPE_CANVAS_GROUP,
+			G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (G_OBJECT_CLASS (object_class),
+		PROP_THEME_COMPLIANCE, g_param_spec_boolean ("theme_compliance",
+			"Theme compliance", "Theme compliance", TRUE,
+			G_PARAM_READWRITE));
+	g_object_class_install_property (G_OBJECT_CLASS (object_class),
+		PROP_USE_THEME, g_param_spec_boolean ("use_theme",
+			"Use theme", "Use theme", FALSE,
+			G_PARAM_READWRITE));
+	g_object_class_install_property (G_OBJECT_CLASS (object_class),
+		PROP_ONLY_FIRST, g_param_spec_boolean ("only_first",
+			"Show only first page", "Show only first page", FALSE,
+			G_PARAM_READWRITE));
 }
 
 GnomePrintContext *
 gnome_print_preview_new_full (GnomePrintConfig *config, GnomeCanvas *canvas,
 			      const gdouble *transform, const ArtDRect *region)
 {
-	GnomePrintPreview *preview;
-	GnomePrintContext *ctx;
-	GnomeCanvasGroup *group;
-	gint ret;
-	
+	GnomeCanvasItem *group;
+
 	g_return_val_if_fail (config != NULL, NULL);
 	g_return_val_if_fail (canvas != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_CANVAS (canvas), NULL);
 	g_return_val_if_fail (transform != NULL, NULL);
 	g_return_val_if_fail (region != NULL, NULL);
 
-	ctx = g_object_new (GNOME_TYPE_PRINT_PREVIEW, NULL);
-	ret = gnome_print_context_construct (ctx, config);
-	if (ret != GNOME_PRINT_OK) {
-		g_object_unref (ctx);
-		g_warning ("Could not construct the GnomePrintPreview context\n");
-		return NULL;
-	}
-	preview = GNOME_PRINT_PREVIEW (ctx);
+	gnome_canvas_set_scroll_region (canvas, region->x0, region->y0,
+									region->x1, region->y1);
+	group = gnome_canvas_item_new (gnome_canvas_root (canvas),
+									GNOME_TYPE_CANVAS_GROUP, NULL);
+	gnome_canvas_item_affine_absolute (group, transform);
 
-	g_object_ref (G_OBJECT (canvas));
-	preview->canvas = canvas;
-
-	gnome_canvas_set_scroll_region (canvas, region->x0, region->y0, region->x1, region->y1);
-
-	preview->priv->root = gnome_canvas_item_new (gnome_canvas_root (preview->canvas),
-						     GNOME_TYPE_CANVAS_GROUP,
-						     "x", 0.0, "y", 0.0, NULL);
-
-	preview->priv->page = gnome_canvas_item_new (gnome_canvas_root (preview->canvas),
-						     GNOME_TYPE_CANVAS_GROUP,
-						     "x", 0.0, "y", 0.0, NULL);
-
-	g_signal_connect (G_OBJECT (preview->priv->page), "destroy",
-			  (GCallback) clear_val, &preview->priv->page);
-	g_signal_connect (G_OBJECT (preview->priv->root), "destroy",
-			  (GCallback) clear_val, &preview->priv->root);
-
-	/* Setup base group */
-	group = GNOME_CANVAS_GROUP (preview->priv->page);
-	gp_gc_set_data (preview->priv->gc, group);
-
-	gnome_canvas_item_affine_absolute (preview->priv->page, transform);
-
-	return ctx;
+	return g_object_new (GNOME_TYPE_PRINT_PREVIEW, "group", group, NULL);
 }
 
 /**
@@ -534,10 +541,10 @@ gnome_print_preview_new (GnomePrintConfig *config, GnomeCanvas *canvas)
 		bbox.y0 = 0.0;
 		bbox.x1 = 21.0 * (72.0 / 2.54);
 		bbox.y1 = 29.7 * (72.0 / 2.54);
-		if (gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAPER_WIDTH, &bbox.x1, &unit)) {
+		if (gnome_print_config_get_length (config, (const guchar *) GNOME_PRINT_KEY_PAPER_WIDTH, &bbox.x1, &unit)) {
 			gnome_print_convert_distance (&bbox.x1, unit, GNOME_PRINT_PS_UNIT);
 		}
-		if (gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAPER_HEIGHT, &bbox.y1, &unit)) {
+		if (gnome_print_config_get_length (config, (const guchar *) GNOME_PRINT_KEY_PAPER_HEIGHT, &bbox.y1, &unit)) {
 			gnome_print_convert_distance (&bbox.y1, unit, GNOME_PRINT_PS_UNIT);
 		}
 	}
@@ -569,7 +576,7 @@ gnome_print_preview_get_type (void)
 			NULL, NULL,
 			sizeof (GnomePrintPreview),
 			0,
-			(GInstanceInitFunc) instance_init
+			(GInstanceInitFunc) NULL
 		};
 		preview_type = g_type_register_static (GNOME_TYPE_PRINT_CONTEXT, "GnomePrintPreview", &preview_info, 0);
 	}
