@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  gpa-transport-selector.c: A print transport selector
  *
@@ -37,6 +37,7 @@
 #include <libgnomeprint/gnome-print-config.h>
 #include <libgnomeprint/private/gpa-node-private.h>
 #include <libgnomeprint/private/gpa-key.h>
+#include <libgnomeprint/gnome-print-transport.h>
 
 static void gpa_transport_selector_class_init (GPATransportSelectorClass *klass);
 static void gpa_transport_selector_init (GPATransportSelector *selector);
@@ -93,7 +94,7 @@ gpa_transport_selector_init (GPATransportSelector *ts)
 
 	hbox = (GtkBox *) gtk_hbox_new (FALSE, 4);
 
-	ts->menu = gtk_option_menu_new ();
+	ts->combo = gtk_combo_box_new ();
 	ts->file_name   = g_strdup ("");
 	ts->file_name_force = FALSE;
 	ts->file_name_label   = gtk_label_new ("");
@@ -105,7 +106,7 @@ gpa_transport_selector_init (GPATransportSelector *ts)
 	g_signal_connect (G_OBJECT (ts->custom_entry), "changed", (GCallback)
 			  gpa_transport_selector_custom_entry_changed_cb, ts);
 
-	gtk_box_pack_start (hbox, ts->menu,         FALSE, FALSE, 0);
+	gtk_box_pack_start (hbox, GTK_WIDGET (ts->combo),	FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, ts->file_button,   FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, ts->file_name_label,  FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, ts->custom_entry, FALSE, FALSE, 0);
@@ -256,10 +257,6 @@ gpa_transport_selector_file_button_clicked_cb (GtkButton *button, GPATransportSe
 	{	
 		GtkFileFilter *filter;
 
-		filter = gtk_file_filter_new ();
-		gtk_file_filter_set_name (filter, _("All Files"));
-		gtk_file_filter_add_pattern (filter, "*");
-		gtk_file_chooser_add_filter (ts->file_selector, filter);
  
 		filter = gtk_file_filter_new ();
 		gtk_file_filter_set_name (filter, "PDF Files");
@@ -270,17 +267,28 @@ gpa_transport_selector_file_button_clicked_cb (GtkButton *button, GPATransportSe
 		gtk_file_filter_set_name (filter, "Postscript Files");
 		gtk_file_filter_add_pattern (filter, "*.ps");
 		gtk_file_chooser_add_filter (ts->file_selector, filter);
+
+		filter = gtk_file_filter_new ();
+		gtk_file_filter_set_name (filter, _("All Files"));
+		gtk_file_filter_add_pattern (filter, "*");
+		gtk_file_chooser_add_filter (ts->file_selector, filter);
+		gtk_file_chooser_set_filter (ts->file_selector, filter);
 	}
 
 	gtk_file_chooser_unselect_all (ts->file_selector);
 	
 	filename = g_filename_from_utf8 (ts->file_name, -1,
 					 &bytes_read, &bytes_written, NULL);
-	if (filename != NULL) {
+	if ((filename != NULL) && g_path_is_absolute (filename))
+		gtk_file_chooser_set_filename (ts->file_selector,
+					       filename);
+	else if (ts->file_name != NULL) {
 		gtk_file_chooser_set_current_name (ts->file_selector, 
-						   filename);
-		g_free (filename);
+						   ts->file_name);
 	}
+	
+	if (filename != NULL)
+		g_free (filename);
    
 	g_signal_connect (ts->file_selector, "response",
 			  G_CALLBACK (gpa_transport_selector_file_selected_cb),
@@ -351,11 +359,15 @@ gpa_transport_selector_update_widgets (GPATransportSelector *ts)
 }
 
 static void
-gpa_transport_selector_item_activate_cb (GtkWidget *item, GPATransportSelector *ts)
+gpa_transport_selector_item_activate_cb (GPATransportSelector *ts)
 {
+	GtkTreeIter   iter;
 	GPANode *node;
 
-	node =  g_object_get_data (G_OBJECT (item), "node");
+	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ts->combo), &iter);
+	gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (ts->combo)),
+		&iter, 1, &node, -1);
+
 	ts->updating = TRUE;
 	gpa_node_set_value (ts->node, gpa_node_id (node));
 	ts->updating = FALSE;
@@ -364,67 +376,63 @@ gpa_transport_selector_item_activate_cb (GtkWidget *item, GPATransportSelector *
 }
 
 static void
-gpa_transport_selector_rebuild_menu (GPATransportSelector *ts)
+gpa_transport_selector_rebuild_combo (GPATransportSelector *ts)
 {
-	GtkWidget *menu, *item;
-	GPANode *option, *child;
-	guchar *def = NULL;
+	GtkListStore	*model;
+	GtkTreeIter	 iter;
+	GPANode *child, *next, *mod, *option = NULL;
+	guchar *mod_name, *trans_name;
 	gint pos = 0;
 	gint sel = -1;
 
-	menu = gtk_menu_new ();
-	gtk_widget_show (menu);
-
+	model = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_OBJECT);
 	if (ts->node != NULL) {
+		guchar *def = gpa_node_get_value (ts->node);
 		option = GPA_KEY (ts->node)->option;
-		def = gpa_node_get_value (ts->node);
 		
-		child = gpa_node_get_child (option, NULL);
-		while (child) {
-			const guchar *id;
-			guchar *name;
-			
-			name = gpa_node_get_value (child);
-			item = gtk_menu_item_new_with_label (name);
-			g_free (name);
-			
-			id = gpa_node_id (child);
-			
-			g_signal_connect 
-				(G_OBJECT (item), "activate", 
-				 (GCallback) 
-				 gpa_transport_selector_item_activate_cb, ts);
-			g_object_set_data_full 
-				(G_OBJECT (item), "node", child, 
-				 (GtkDestroyNotify) gpa_node_unref);
-			gtk_widget_show (item);
-			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-			if (GPA_NODE_ID_COMPARE (child, def))
-				sel = pos;
-			
-			pos++;
-			child = gpa_node_get_child (option, child);
+		for (child = gpa_node_get_child (option, NULL); child != NULL ; child = next) {
+			mod = gpa_node_get_child_from_path (child, "Module");
+			if (mod != NULL) {
+				mod_name = gpa_node_get_value (mod);
+				if (gnome_print_transport_exists_by_name (mod_name)) {
+					trans_name = gpa_node_get_value (child);
+					gtk_list_store_append (model, &iter);
+					gtk_list_store_set (model, &iter,
+							    0, trans_name,
+							    1, child,
+							    -1);
+					if (GPA_NODE_ID_COMPARE (child, def))
+						sel = pos;
+					pos++;
+					g_free (trans_name);
+				}
+				g_free (mod_name);
+			}
+
+			next = gpa_node_get_child (option, child);
+			gpa_node_unref (child);
 		}
+
 		if (sel == -1) {
-			g_warning ("rebuild_menu_cb, could not set value "
-				   "of %s to %s",
+			g_warning ("gpa_transport_selector_rebuild_combo, could not set value of %s to %s",
 				   gpa_node_id (option), def);
 			sel = 0;
 		}
-	}
-	if (pos < 1) {
-		item = gtk_menu_item_new_with_label 
-			(_("No options are defined"));
-		gtk_widget_set_sensitive (item, FALSE);
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+		if (def)
+			g_free (def);
 	}
 
-	gtk_widget_show (menu);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (ts->menu), menu);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (ts->menu), sel);
-	if (def)
-		g_free (def);
+	if (pos <= 1)
+		gtk_widget_hide (ts->combo);
+	else
+		gtk_widget_show (ts->combo);
+
+	ts->updating = TRUE;
+	gtk_combo_box_set_model (GTK_COMBO_BOX (ts->combo), GTK_TREE_MODEL (model));
+	if (pos > 0)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (ts->combo), sel);
+	ts->updating = FALSE;
 
 	gpa_transport_selector_update_widgets (ts);
 }
@@ -445,7 +453,7 @@ gpa_transport_selector_config_modified_cb (GPANode *node, guint flags, GPATransp
 {
 	gpa_transport_selector_disconnect (ts);
 	gpa_transport_selector_connect (ts);
-	gpa_transport_selector_rebuild_menu (ts);
+	gpa_transport_selector_rebuild_combo (ts);
 }
 
 static void
@@ -454,13 +462,14 @@ gpa_transport_selector_node_modified_cb (GPANode *node, guint flags, GPATranspor
 	if (ts->updating)
 		return;
 	
-	gpa_transport_selector_rebuild_menu (ts);
+	gpa_transport_selector_rebuild_combo (ts);
 }
 
 static gint
 gpa_transport_selector_construct (GPAWidget *gpaw)
 {
 	GPATransportSelector *ts;
+        GtkCellRenderer *renderer;
 
 	ts = GPA_TRANSPORT_SELECTOR (gpaw);
 	ts->config  = GNOME_PRINT_CONFIG_NODE (gpaw->config);
@@ -468,7 +477,16 @@ gpa_transport_selector_construct (GPAWidget *gpaw)
 					       gpa_transport_selector_config_modified_cb, ts);
 
 	gpa_transport_selector_connect (ts);
-	gpa_transport_selector_rebuild_menu (ts);
+	gpa_transport_selector_rebuild_combo (ts);
+
+	g_signal_connect_swapped (G_OBJECT (ts->combo),
+		"changed", 
+		G_CALLBACK (gpa_transport_selector_item_activate_cb), ts);
+        renderer = gtk_cell_renderer_text_new ();
+        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ts->combo),
+		renderer, TRUE);
+        gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (ts->combo),
+		renderer, "text", 0, NULL);
 
 	return TRUE;
 }
@@ -486,8 +504,11 @@ gpa_transport_selector_check_consistency_real (GPATransportSelector *ts)
 	if (backend == NULL)
 		return TRUE;
 
-	if (strcmp (backend, "file") != 0)
+	if (strcmp (backend, "file") != 0) {
+		g_free (backend);
 		return TRUE;
+	}
+	g_free (backend);
 	
 	if (ts->file_name_force)
 		return TRUE;
